@@ -1,45 +1,48 @@
 # -*- coding: utf-8 -*-
 
-"""Extractor for images in a generic web page."""
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+
+"""Generic information extractor"""
 
 from .common import Extractor, Message
 from .. import config, text
-import re
 import os.path
+import re
 
 
 class GenericExtractor(Extractor):
     """Extractor for images in a generic web page."""
-
     category = "generic"
-    directory_fmt = ("{category}", "{pageurl}")
+    directory_fmt = ("{category}", "{subcategory}", "{path}")
     archive_fmt = "{imageurl}"
 
     # By default, the generic extractor is disabled
     # and the "g(eneric):" prefix in url is required.
     # If the extractor is enabled, make the prefix optional
-    pattern = r"(?ix)(?P<generic>g(?:eneric)?:)"
+    pattern = r"(?i)(?P<generic>g(?:eneric)?:)"
     if config.get(("extractor", "generic"), "enabled"):
         pattern += r"?"
 
     # The generic extractor pattern should match (almost) any valid url
     # Based on: https://tools.ietf.org/html/rfc3986#appendix-B
-    pattern += r"""
-        (?P<scheme>https?://)?          # optional http(s) scheme
-        (?P<domain>[-\w\.]+)            # required domain
-        (?P<path>/[^?#]*)?              # optional path
-        (?:\?(?P<query>[^#]*))?         # optional query
-        (?:\#(?P<fragment>.*))?         # optional fragment
-        """
+    pattern += (
+        r"(?P<scheme>https?://)?"          # optional http(s) scheme
+        r"(?P<domain>[-\w\.]+)"            # required domain
+        r"(?P<path>/[^?#]*)?"              # optional path
+        r"(?:\?(?P<query>[^#]*))?"         # optional query
+        r"(?:\#(?P<fragment>.*))?"         # optional fragment
+    )
+    example = "generic:https://www.nongnu.org/lzip/"
 
     def __init__(self, match):
-        """Init."""
+        self.subcategory = match.group('domain')
         Extractor.__init__(self, match)
 
         # Strip the "g(eneric):" prefix
         # and inform about "forced" or "fallback" mode
         if match.group('generic'):
-            self.log.info("Forcing use of generic information extractor.")
             self.url = match.group(0).partition(":")[2]
         else:
             self.log.info("Falling back on generic information extractor.")
@@ -50,13 +53,15 @@ class GenericExtractor(Extractor):
             self.scheme = match.group('scheme')
         else:
             self.scheme = 'https://'
-            self.url = self.scheme + self.url
+            self.url = text.ensure_http_scheme(self.url, self.scheme)
+
+        self.path = match.group('path')
 
         # Used to resolve relative image urls
         self.root = self.scheme + match.group('domain')
 
     def items(self):
-        """Get page, extract metadata & images, yield them in suitable messages.
+        """Get page, extract metadata & images, yield them in suitable messages
 
         Adapted from common.GalleryExtractor.items()
 
@@ -71,7 +76,6 @@ class GenericExtractor(Extractor):
             pass
         images = enumerate(imgs, 1)
 
-        yield Message.Version, 1
         yield Message.Directory, data
 
         for data["num"], (url, imgdata) in images:
@@ -85,29 +89,33 @@ class GenericExtractor(Extractor):
 
     def metadata(self, page):
         """Extract generic webpage metadata, return them in a dict."""
-        data = {}
-        data['pageurl'] = self.url
-        data['title'] = text.extr(page, '<title>', "</title>")
-        data['description'] = text.extr(
-            page, '<meta name="description" content="', '"')
-        data['keywords'] = text.extr(
-            page, '<meta name="keywords" content="', '"')
-        data['language'] = text.extr(
-            page, '<meta name="language" content="', '"')
-        data['name'] = text.extr(
-            page, '<meta itemprop="name" content="', '"')
-        data['copyright'] = text.extr(
-            page, '<meta name="copyright" content="', '"')
-        data['og_site'] = text.extr(
-            page, '<meta property="og:site" content="', '"')
-        data['og_site_name'] = text.extr(
-            page, '<meta property="og:site_name" content="', '"')
-        data['og_title'] = text.extr(
-            page, '<meta property="og:title" content="', '"')
-        data['og_description'] = text.extr(
-            page, '<meta property="og:description" content="', '"')
+        data = {
+            "title"         : text.extr(
+                page, "<title>", "</title>"),
+            "description"   : text.extr(
+                page, '<meta name="description" content="', '"'),
+            "keywords"      : text.extr(
+                page, '<meta name="keywords" content="', '"'),
+            "language"      : text.extr(
+                page, '<meta name="language" content="', '"'),
+            "name"          : text.extr(
+                page, '<meta itemprop="name" content="', '"'),
+            "copyright"     : text.extr(
+                page, '<meta name="copyright" content="', '"'),
+            "og_site"       : text.extr(
+                page, '<meta property="og:site" content="', '"'),
+            "og_site_name"  : text.extr(
+                page, '<meta property="og:site_name" content="', '"'),
+            "og_title"      : text.extr(
+                page, '<meta property="og:title" content="', '"'),
+            "og_description": text.extr(
+                page, '<meta property="og:description" content="', '"'),
 
-        data = {k: text.unescape(data[k]) for k in data if data[k] != ""}
+        }
+
+        data = {k: text.unescape(v) for k, v in data.items() if v}
+        data["path"] = self.path.replace("/", "")
+        data["pageurl"] = self.url
 
         return data
 
@@ -136,11 +144,13 @@ class GenericExtractor(Extractor):
         image urls; this pattern matches only the first url; remaining urls
         will be matched by the "imageurl_pattern_ext" pattern below.
         """
-        imageurl_pattern_src = r"""(?ix)
-            <(?:img|video|source)\s.*?      # <img>, <video> or <source>
-            src(?:set)?=["']?               # src or srcset attributes
-            (?P<URL>[^"'\s>]+)              # url
-            """
+
+        imageurl_pattern_src = (
+            r"(?i)"
+            r"<(?:img|video|source)\s[^>]*"    # <img>, <video> or <source>
+            r"src(?:set)?=[\"']?"              # src or srcset attributes
+            r"(?P<URL>[^\"'\s>]+)"             # url
+        )
 
         """
         2: Look anywhere for urls containing common image/video extensions
@@ -150,16 +160,17 @@ class GenericExtractor(Extractor):
         https://en.wikipedia.org/wiki/List_of_file_formats
 
         Compared to the "pattern" class variable, here we must exclude also
-        other special characters (space, ", ', >), since we are looking for
+        other special characters (space, ", ', <, >), since we are looking for
         urls in html tags.
         """
 
-        imageurl_pattern_ext = r"""(?ix)
-            (?:[^?&#"'>\s]+)                    # anything until dot+extension
-            \.(?:jpe?g|jpe|png|gif
-                 |web[mp]|mp4|mkv|og[gmv]|opus) # dot + image/video extensions
-            (?:[^"'>\s]*)?                      # optional query and fragment
-            """
+        imageurl_pattern_ext = (
+            r"(?i)"
+            r"(?:[^?&#\"'>\s]+)"           # anything until dot+extension
+                                           # dot + image/video extensions
+            r"\.(?:jpe?g|jpe|png|gif|web[mp]|mp4|mkv|og[gmv]|opus)"
+            r"(?:[^\"'<>\s]*)?"            # optional query and fragment
+        )
 
         imageurls_src = re.findall(imageurl_pattern_src, page)
         imageurls_ext = re.findall(imageurl_pattern_ext, page)
@@ -199,7 +210,7 @@ class GenericExtractor(Extractor):
                 absimageurls.append(self.baseurl + '/' + u)
 
         # Remove duplicates
-        absimageurls = set(absimageurls)
+        absimageurls = dict.fromkeys(absimageurls)
 
         # Create the image metadata dict and add imageurl to it
         # (image filename and extension are added by items())
