@@ -153,7 +153,7 @@ class KemonopartyExtractor(Extractor):
                     file["type"] = "archive"
                     if archives:
                         try:
-                            data = self.api.posts_archives(file["hash"])
+                            data = self.api.file(file["hash"])
                             data.update(file)
                             post_archives.append(data)
                         except Exception as exc:
@@ -319,12 +319,9 @@ class KemonopartyUserExtractor(KemonopartyExtractor):
     def posts(self):
         _, _, service, creator_id, query = self.groups
         params = text.parse_query(query)
-        if params.get("tag"):
-            return self.api.creator_tagged_posts(
-                service, creator_id, params.get("tag"), params.get("o"))
-        else:
-            return self.api.creator_posts(
-                service, creator_id, params.get("o"), params.get("q"))
+        return self.api.creator_posts_legacy(
+            service, creator_id,
+            params.get("o"), params.get("q"), params.get("tag"))
 
 
 class KemonopartyPostsExtractor(KemonopartyExtractor):
@@ -524,18 +521,19 @@ class KemonoAPI():
         params = {"q": query, "o": offset, "tag": tags}
         return self._pagination(endpoint, params, 50, "posts")
 
-    def posts_archives(self, file_hash):
-        endpoint = "/posts/archives/" + file_hash
-        return self._call(endpoint)["archive"]
+    def file(self, file_hash):
+        endpoint = "/file/" + file_hash
+        return self._call(endpoint)
 
     def creator_posts(self, service, creator_id, offset=0, query=None):
         endpoint = "/{}/user/{}".format(service, creator_id)
         params = {"q": query, "o": offset}
         return self._pagination(endpoint, params, 50)
 
-    def creator_tagged_posts(self, service, creator_id, tags, offset=0):
+    def creator_posts_legacy(self, service, creator_id,
+                             offset=0, query=None, tags=None):
         endpoint = "/{}/user/{}/posts-legacy".format(service, creator_id)
-        params = {"o": offset, "tag": tags}
+        params = {"o": offset, "tag": tags, "q": query}
         return self._pagination(endpoint, params, 50, "results")
 
     def creator_announcements(self, service, creator_id):
